/* TestLib for C++ v. 0.3.1
 *
 * Yury Petrov aka Burunduk2, May 18, 2008 - February 22, 2009
 * mailto:jpsbur@gmail.com
 *
 * Comments, bug reports, suggestions and so on are
 * appreciated
 *
 * modifications:
 *  2009.02.22:
 *    - 'inputStream' class 'error' method bugfix
 */

// Note that TestLib works only with 8-bit encodings.
// Also note that using non-ASCII characters is not
// advised though supported.

#ifndef _testlib_
#define _testlib_

// Compiler-dependent features

#ifdef __GNUC__
#  define LINUX
#else
#  define WIN
#endif

#ifdef LINUX
#  define LLD "%lld"
#  define LLU "%llu"
#else
#  define LLD "%I64d"
#  define LLU "%I64u"
#endif

#include <cstdio>
#include <cstring>
#include <cstdlib>
#include <string>

using namespace std;

namespace testlib {

// inStr is a base class for all input streams
class inStr {
public:
  bool ok;
  inStr() {
  }
  // Should return -1 on end of stream
  virtual int gc() = 0;
};

// inFile is an inStr implementation for files
class inFile: public inStr {
  FILE *f;
public:
  inFile( string fn ) {
    f = fopen(fn.c_str(), "r");
    ok = f != 0;
  }
  inFile( FILE *_f ) {
    f = _f;
    ok = f != 0;
  }
  ~inFile() {
    if (f)
      fclose(f);
  }
  int gc() {
    return fgetc(f);
  }
};

// inString is an inStr implementation for strings
class inString: public inStr {
private:
  string s;
  int i;
public:
  inString() {
    ok = true;
    s = "";
    i = 0;
  }
  inString( string _s ) {
    ok = true;
    s = _s;
    i = 0;
  }
  int gc() {
    if (i == (int)s.length())
      return -1;
    return s[i++];
  }
};

// charSet is an abstraction for set of characters.
// It supports 1-byte characters.
// It might implicitly support EOF character if it is
// passed as -1 (streamParser does not use this feature).

class charSet {
private:
  static const int m = 256 / 32;
  unsigned *is;
public:
  charSet( char c ) {
    is = new unsigned[m];
    memset(is, 0, sizeof(unsigned) * m);
    is[(unsigned)c >> 5] |= 1 << ((unsigned)c & 31);
  }
  charSet( string s = "" ) {
    is = new unsigned[m];
    memset(is, 0, sizeof(unsigned) * m);
    for (int i = 0; i < (int)s.length(); i++)
      is[(unsigned)s[i] >> 5] |= 1 << ((unsigned)s[i] & 31);
  }
  charSet( const charSet &c ) {
    is = new unsigned[m];
    memcpy(is, c.is, sizeof(unsigned) * m);
  }
  ~charSet() {
    delete is;
  }
  charSet unite( charSet b ) {
    charSet c;
    for (int i = 0; i < m; i++)
      c.is[i] = is[i] | b.is[i];
    return c;
  }
  void insert( char c ) {
    is[(unsigned)c >> 5] |= 1 << ((unsigned)c & 31);
  }
  charSet intersect( charSet b ) {
    charSet c;
    for (int i = 0; i < m; i++)
      c.is[i] = is[i] & b.is[i];
    return c;
  }
  charSet exclude( charSet b ) {
    charSet c;
    for (int i = 0; i < m; i++)
      c.is[i] = is[i] ^ (is[i] & b.is[i]);
    return c;
  }
  bool contains( char c ) {
    return (is[(unsigned)c >> 5] >> ((unsigned)c & 31)) & 1;
  }
  bool contains( charSet s ) {
    bool res = true;
    for (int i = 0; res && i < m; i++)
      if ((is[i] | s.is[i]) != is[i])
        res = false;
    return res;
  }
};

// Default useful charSets
static charSet ws(" \n\r\t");
static charSet eoln("\n\r");

// Just several useful features

string intToStr( int a ) {
  char buf[99];
  sprintf(buf, "%d", a);
  return buf;
}

string longToStr( long long a ) {
  char buf[99];
  sprintf(buf, LLD, a);
  return buf;
}

string floatToStr( double a, int pr = 5 ) {
  char buf[99];
  sprintf(buf, "%.*lf", pr, a);
  return buf;
}

string shortStr( string s, int l = 15 ) {
  if (l < 3 || (int)s.length() <= 2 * l)
    return s;
  string res = "\"";
  for (int i = 0; i < l; i++)
    res += s[i];
  res += "<...>";
  for (int i = 0; i < l; i++)
    res += s[s.length() - l + i];
  res += "\"";
  return res;
}

// Testing systems return codes mapping (3rd parameter
// of testlib class constructor)

// TestSys by Kitten Computing
#define TS_TestSys 0
// ejudge by cher
#define TS_ejudge  1

static const int ok = 0;
static const int wa = 1;
static const int pe = 2;
static const int fl = 3;

static const int realRet[][4] = {
  { 0, 1, 2, 3 },
  { 0, 4, 5, 17 }
};

class library
{
public:
  virtual void quit( int, string ) = 0;
};

// streamParser may be used for parsing files, strings
// or whatever you create inStr derivative for.
// It supports no buffering at the moment and works not too fast.

class streamParser {
private:
  inStr *str;
  char nc;
  bool empty;
  virtual void error( string ) = 0;
protected:
  library *par;
public:
  streamParser( inStr *_str, library *_par ) {
    if (!_str || !_str->ok) {
      fprintf(stderr, "streamParser error: bad input stream");
      return;
    }
    str = _str;
    par = _par;
    int t = str->gc();
    empty = (t == -1);
    nc = t;
  }
  ~streamParser() {
    delete str;
  }
  char peekChar() {
    if (empty)
      error("No characters left");
    return nc;
  }
  char nextChar() {
    if (empty)
      error("No characters left");
    char t = nc;
    int c = str->gc();
    if (c == -1)
      empty = true;
    nc = c;
    return t;
  }
  bool isEmpty() {
    return empty;
  }
  void skip( charSet s ) {
    while (!isEmpty() && s.contains(peekChar()))
      nextChar();
  }
  string nextWord( charSet before = ws, charSet after = ws ) {
    skip(before);
    string res = "";
    while (!isEmpty() && !after.contains(peekChar()))
      res += nextChar();
    return res;
  }
  int nextInt() {
    skip(ws);
    int res = 0, sign = 1;
    string t;
    if (peekChar() == '-')
      t += '-', sign = -1, nextChar();
    char c = peekChar();
    if (!('0' <= c && c <= '9'))
      error("Integer expected");
    while ('0' <= c && c <= '9') {
      res = res * 10 + c - '0';
      nextChar();
      t += c;
      c = peekChar();
    }
    res *= sign;
    char buf[99];
    sprintf(buf, "%d", res);
    if (t != string(buf))
      error("Integer expected");
    return res;
  }
  long long nextLong() {
    skip(ws);
    long long res = 0, sign = 1;
    string t;
    if (peekChar() == '-')
      t += '-', sign = -1, nextChar();
    char c = peekChar();
    if (!('0' <= c && c <= '9'))
      error("64-bit integer expected");
    while ('0' <= c && c <= '9')
    {
      res = res * 10 + c - '0';
      nextChar();
      t += c;
      c = peekChar();
    }
    res *= sign;
    char buf[99];
    sprintf(buf, LLD, res);
    if (t != string(buf))
      error("64-bit integer expected");
    return res;
  }
  double nextFloat() {
    skip(ws);
    string t;
    bool ok = false;
    char c = peekChar();
    if (c == '-')
      t += nextChar(), c = peekChar();
    while ('0' <= c && c <= '9')
      ok = true, t += nextChar(), c = peekChar();
    if (peekChar() == '.')
      t += nextChar(), c = peekChar();
    while ('0' <= c && c <= '9')
      ok = true, t += nextChar(), c = peekChar();
    if (!ok)
      error("Real number expected");
    double res;
    sscanf(t.c_str(), "%lf", &res);
    return res;
  }
  string nextLine() {
    string res = nextWord(charSet(""), eoln);
    if (!isEmpty())
      nextChar();
    return res;
  }
  bool seekEOF() {
    skip(ws);
    return isEmpty();
  }
};

// Stream parser derivants differ in the way they handle
// internal presentation errors. Input and answer file streams
// return it as 'fail', output file parser returns it as 'pe'

class inStreamParser: public streamParser {
private:
  void error( string cmt ) {
    if (par)
      par->quit(fl, cmt);
  }
public:
  inStreamParser( inStr *_str, library *_par ):
    streamParser(_str, _par) {
  }
};

class outStreamParser: public streamParser {
private:
  void error( string cmt ) {
    if (par)
      par->quit(pe, cmt);
  }
public:
  outStreamParser( inStr *_str, library *_par ):
    streamParser(_str, _par) {
  }
};

class ansStreamParser: public streamParser {
private:
  void error( string cmt ) {
    if (par)
      par->quit(fl, cmt);
  }
public:
  ansStreamParser( inStr *_str, library *_par ):
    streamParser(_str, _par) {
  }
};

class testLib: library {
private:
  int ts;
public:
  FILE *cf;
  inStreamParser *inf;
  outStreamParser *ouf;
  ansStreamParser *ans;

  testLib( int argc = 0, char **argv = 0, int _ts = TS_TestSys ) {
    ts = _ts;
    cf = 0;
    if (argc < 4 || argc > 5) {
      printf("fail: Checker sould be run as the following: "
        "check <in-file> <out-file> <answer-file> [<comment>]\n");
      exit(fl);
    }
    if (argc >= 5)
      cf = fopen(argv[4], "r");
    else
      cf = 0;
    inf = new inStreamParser(new inFile(argv[1]), this);
    ouf = new outStreamParser(new inFile(argv[2]), this);
    ans = new ansStreamParser(new inFile(argv[3]), this);
  }
  int getPCCode( int val ) {
    if (val < 0 || val > 200)
      return fl;
    return 50 + val;
  }
  void quit( int res, string comment = "" ) {
    if ((0 > res || res > 3) && (50 > res || 250 > res))
      quit(fl, "Bad quit code");
    static const char *cmt[5] = { "ok", "wa", "pe", "fail", "pc" };
    printf("%s: %s\n", cmt[res], comment.c_str());
    if (cf) {
      fprintf(cf, "%s: %s\n", cmt[(res < 50) ? res : 4], comment.c_str());
      fclose(cf);
    }
    if (inf)
      delete inf;
    if (ouf)
      delete ouf;
    if (ans)
      delete ans;
    if (res < 50)
      res = realRet[ts][res];
    exit(res);
  }
};

}

#endif

